#---------------------------------------
# Contribution by Robin Mills, San Jose, California
# All rights donated to the open source project Phatch
# This code may be published using a license acceptable to project Phatch
# 

import os
import pyexiv2
import time
import datetime
import xml.dom.minidom

from other import surd

##
# Rational number support
def R(f):
    """R(float) - get a Rational number for a float"""
    s = surd.surd(float(f))
    return pyexiv2.Rational(s.num,s.denom) 

def d(angle):
    """d(any) - get degrees from a number :eg d(33.41) -> 33"""
    return int(angle)

def m(angle):
    """m(any) - get minutes from a number :eg d(33.41) -> 24"""
    return int( angle*60 - d(angle)* 60)

def s(angle):
    """s(any) - get seconds from a number :eg s(33.41) -> 36"""
    return int( angle*3600 - d(angle)*3600 - m(angle)*60 )

#
##


##
# dictionary search (closest match)
def search(dict,target):
    """search(dict,taget) - search for closest match"""
    s    = sorted(dict.keys())
    N    = len(s)
    low  = 0
    high = N-1
    
    while low < high:
        mid = (low + high)/2
        if s[mid] < target:
            low = mid + 1
        else:
            high = mid
    return s[low]       
#
##

## 
# XML functions
def getXMLtimez(phototime,timeshift):
    """getXMLtimez - convert a datetime to an XML formatted date"""
    #
    # phototime = timedate.timedate("2008-03-16 08:52:15")
    # timeshift = seconds
    # -----------------------
    
    timedelta = datetime.timedelta(0,timeshift,0)
    newtime = phototime + timedelta 
    return newtime.strftime('%Y-%m-%dT%H:%M:%SZ') ;

def getText(nodelist,):
    """getText(nodeList) - return the text in nodelist"""
    rc = ""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

def getNodeValue(node):
    """getNodeValue(node) - return the value of a node"""
    return getText(node.childNodes)

def handleTRKPT(trkpt,timedict,ns):
    """handleTRKPT"""
    ele  = getNodeValue(trkpt.getElementsByTagNameNS(ns,"ele" )[0]) if ns else getNodeValue(trkpt.getElementsByTagName("ele" )[0])
    time = getNodeValue(trkpt.getElementsByTagNameNS(ns,"time")[0]) if ns else getNodeValue(trkpt.getElementsByTagName("time")[0])
    lat  = trkpt.getAttributeNS(ns,"lat") if ns else trkpt.getAttribute("lat") 
    lon  = trkpt.getAttributeNS(ns,"lon") if ns else trkpt.getAttribute("lon")
    # Garmin .gpx doesn't use a ns on the lat and lon attributes!  Garmin bug?
    if not lat:
        lat = trkpt.getAttribute("lat") 
    if not lon:
        lon = trkpt.getAttribute("lon") 
      
    # print "lat, lon = %s %s ele,time = %s %s" % ( lat,lon  , ele,time )
    timedict[time] = [ ele,lat,lon ]

def handleTRKSEG(trkseg,timedict,ns):
    """handleTRKSEG"""
    trkpts =     trkseg.getElementsByTagNameNS(ns,"trkpt") if ns else trkseg.getElementsByTagName("trkpt")
    for trkpt in trkpts:
        handleTRKPT(trkpt,timedict,ns)

def handleTRK(trk,timedict,ns):
    """handleTRK"""
    trksegs = trk.getElementsByTagNameNS(ns,"trkseg") if ns else trk.getElementsByTagName("trkseg")
    for trkseg in trksegs:
        handleTRKSEG(trkseg,timedict,ns)

def handleGPX(gpx,timedict,ns):
    """handleGPSX"""
    trks =    gpx.getElementsByTagNameNS(ns,"trk") if ns else gpx.getElementsByTagName("trk")
    for trk in trks:
        handleTRK(trk,timedict,ns)
# 
##

########################################
## BEGIN    GPS module API      BEGIN ##

##
# this code is heading for module core.lib.gps
def readGPX(gpx_file):
    """readGPX(string) - get a dictionary of time/position information  """
    timedict = {}
    print "readGPX = " + gpx_file
    file     = open(gpx_file,"r")
    data     = file.read(os.path.getsize(gpx_file))
    print "reading ",gpx_file
    file.close()
    dom = xml.dom.minidom.parseString(data)

    # read the XML with and without the namepace
    handleGPX(dom,timedict,False)
    ns = 'http://www.topografix.com/GPX/1/1'
    handleGPX(dom,timedict,ns)
    
    return timedict
    
##
# returns the metadata dictionary for given exif date eg 'Exif.Image.DateTime'
def getMetaData(dateString,timedict,timeshift,path,firstTime):
    """getMetaData(float) - get a dictionary of changes to the meta data
       dateString - EXIF date format string /* in */
       timeshift  - delta between GMT and local time (seconds.  Positive to West)
       path       - path to the image (only for stdout reporting)
       firstTime  - only used for reporting
    """
    stamp       = str(getXMLtimez(dateString,timeshift))

    timestamp   = search(timedict,stamp) 
    data        = timedict[timestamp]
    ele         = float(data[0])
    lat         = float(data[1])
    lon         = float(data[2])

    latR    = 'N'
    lonR    = 'E'
    eleR    =  0
    if lat  < 0:
        lat = -lat
        latR= 'S'
    if lon  < 0:
        lon = -lon
        lonR= 'W'
    sele = "%6.1f" % (ele)
    if ele  < 0:
        ele = -ele
        eleR= 1
    if firstTime:
        print "camera time          nearest gps          latitude   longitude     elev photofile"
        firstTime = False
    slat = "%02d.%02d'" '%02d"%s' % (d(lat),m(lat),s(lat),latR )
    slon = "%02d.%02d'" '%02d"%s' % (d(lon),m(lon),s(lon),lonR )
    print  "%s %s %s %s %s %s" % ( stamp,timestamp,slat,slon,sele,path )
    
    # get Rational number for ele
    # don't know why R(ele) is causing trouble!
    # it might be that the denominator is overflowing 32 bits!
    # and this would also import lat and lon
    rele = pyexiv2.Rational(int(ele*10.0),10)
    
    
    ##
    # create and return the dictionary of tags to be added to the image
    metaData = {}

    metaData['Exif.GPSInfo.GPSAltitude'            ] = rele
    metaData['Exif.GPSInfo.GPSAltitudeRef'         ] = eleR
    metaData['Exif.GPSInfo.GPSDateStamp'           ] = stamp
    metaData['Exif.GPSInfo.GPSLatitude'            ] = [R(d(lat)),R(m(lat)),R(s(lat))]
    metaData['Exif.GPSInfo.GPSLatitudeRef'         ] = latR
    metaData['Exif.GPSInfo.GPSLongitude'           ] = [R(d(lon)),R(m(lon)),R(s(lon))]
    metaData['Exif.GPSInfo.GPSLongitudeRef'        ] = lonR
    metaData['Exif.GPSInfo.GPSMapDatum'            ] = 'WGS-84'
    metaData['Exif.GPSInfo.GPSProcessingMethod'    ] = '65 83 67 73 73 0 0 0 72 89 66 82 73 68 45 70 73 88 ' 
    metaData['Exif.GPSInfo.GPSTimeStamp'           ] = [R(10),R(20),R(30)]
    metaData['Exif.GPSInfo.GPSVersionID'           ] = '2 2 0 0'
    
    return metaData

##
##  END     GPS module API       END  ##
########################################

